@@ -241,7 +241,7 @@ setMethod("[", c(OPMS, "ANY", "ANY", "ANY"), function(x, i, j, k, ...,
     y <- x@plates
   else {
     y <- x@plates[i]
-    if (any(bad <- vapply(y, is.null, logical(1L)))) {
+    if (any(bad <- vapply(y, is.null, NA))) {
       warning("plate indexes partially out of range")
       y <- y[!bad]
     }
@@ -435,7 +435,7 @@ setMethod("max", OPM, function(x, ..., na.rm = FALSE) {
 }, sealed = SEALED)
 
 setMethod("max", OPMS, function(x, ..., na.rm = FALSE) {
-  max(vapply(x@plates, FUN = max, numeric(1L), ..., na.rm = na.rm),
+  max(vapply(x@plates, FUN = max, 1, ..., na.rm = na.rm),
     na.rm = na.rm)
 }, sealed = SEALED)
 
@@ -480,7 +480,7 @@ setMethod("minmax", OPM, function(x, ..., na.rm = FALSE) {
 }, sealed = SEALED)
 
 setMethod("minmax", OPMS, function(x, ..., na.rm = FALSE) {
-  min(vapply(x@plates, FUN = minmax, numeric(1L), ..., na.rm = na.rm))
+  min(vapply(x@plates, FUN = minmax, 1, ..., na.rm = na.rm))
 }, sealed = SEALED)
 
 
@@ -602,7 +602,9 @@ setMethod("seq", OPMS, function(...) {
 #' Optionally the full substrate names can be added in parentheses or brackets
 #' or used instead of the coordinate, and trimmed to a given length.
 #'
-#' @param object \code{\link{OPM}} object.
+#' @param object \code{\link{OPM}} object, \code{\link{OPMS}} object or well
+#'   name or index. If missing, defaults to the selection of all possible
+#'   wells.
 #' @param full Logical scalar. Return the full names of the wells (if available)
 #'   or just their coordinates on the plate? The following arguments have no
 #'   effect if \code{full} is \code{FALSE}.
@@ -625,12 +627,23 @@ setMethod("seq", OPMS, function(...) {
 #' @param downcase Logical scalar indicating whether full names should be
 #'   (carefully) converted to lower case. This uses \code{\link{substrate_info}}
 #'   in \sQuote{downcase} mode; see there for details.
+#' @param plate Name of the plate type. Several ones can be given unless
+#'   \code{object} is of class \code{\link{OPM}} or \code{\link{OPMS}}.
+#'   \code{\link{plate_type}} is applied before searching for the substrate
+#'   names, and partial matching is allowed.
 #' @param ... Optional arguments passed between the methods.
 #' @return Character vector.
 #' @export
 #' @family getter-functions
 #' @seealso base::strtrim base::abbreviate
 #' @keywords attribute
+#' @details The purpose of the \code{\link{OPM}} and \code{\link{OPMS}} methods
+#'   should be obvious. The default method is intended for providing a quick
+#'   overview of the substrates contained in one to several plates if
+#'   \code{full} is \code{TRUE}. If \code{full} is \code{FALSE}, it can be used
+#'   to study the effect of the well-index translation and well-name
+#'   normalization approaches as used by \pkg{opm}, particularly by the
+#'   subsetitting methods (see \code{\link{[}}).
 #' @note Do not confuse this with \code{\link{well}}.
 #' @examples
 #'
@@ -648,20 +661,58 @@ setMethod("seq", OPMS, function(...) {
 #' # wells are guaranteed to be uniform within OPMS objects
 #' stopifnot(identical(x, xx))
 #'
+#' # default method
+#' x <- c("A01", "B10")
+#' (y <- wells(x, plate = "PM1"))
+#' stopifnot(nchar(y) > nchar(x))
+#' (z <- wells(x, plate = "PM1", in.parens = TRUE))
+#' stopifnot(nchar(z) > nchar(y))
+#' # formula yields same result (except for row names)
+#' stopifnot(y == wells(~ c(A01, B10), plate = "PM1"))
+#' # using a sequence of well coordinates
+#' stopifnot(nrow(wells(~ C02:C06)) == 5) # well sequence
+#' stopifnot(nrow(wells(plate = "PM1")) == 96) # all wells by default
+#'
 setGeneric("wells", function(object, ...) standardGeneric("wells"))
 
 setMethod("wells", OPM, function(object, full = FALSE, in.parens = TRUE,
     max = 100L, brackets = FALSE, clean = TRUE, word.wise = FALSE,
-    paren.sep = " ", downcase = FALSE) {
+    paren.sep = " ", downcase = FALSE, plate = plate_type(object)) {
   result <- setdiff(colnames(measurements(object)), HOUR)
-  if (full)
-    map_well_names(result, plate_type(object), in.parens = in.parens,
+  if (L(full))
+    map_well_names(result, L(plate), in.parens = in.parens,
       max = max, brackets = brackets, clean = clean, word.wise = word.wise,
       paren.sep = paren.sep, downcase = downcase)
   else
     result
 }, sealed = SEALED)
 
+setMethod("wells", "ANY", function(object, full = TRUE, in.parens = FALSE,
+    max = 100L, brackets = FALSE, clean = TRUE, word.wise = FALSE,
+    paren.sep = " ", downcase = FALSE, plate = "PM01") {
+  result <- well_index(object, rownames(WELL_MAP))
+  if (!is.character(result))
+    result <- rownames(WELL_MAP)[result]
+  result <- do.call(cbind, rep.int(list(result), length(plate)))
+  pos <- pmatch(plate_type(plate), colnames(WELL_MAP))
+  colnames(result) <- plate
+  if (is.character(object))
+    rownames(result) <- object
+  if (!L(full))
+    return(result)
+  for (i in which(!is.na(pos)))
+    result[, i] <- map_well_names(result[, i], colnames(WELL_MAP)[pos[i]],
+      in.parens = in.parens, max = max, brackets = brackets, clean = clean,
+      word.wise = word.wise, paren.sep = paren.sep, downcase = downcase)
+  for (i in which(is.na(pos)))
+    result[, i] <- NA_character_
+  result
+}, sealed = SEALED)
+
+setMethod("wells", "missing", function(object, ...) {
+  wells(object = TRUE, ...)
+}, sealed = SEALED)
+
 
 ################################################################################
 ################################################################################
@@ -676,7 +727,7 @@ setMethod("wells", OPM, function(object, full = FALSE, in.parens = TRUE,
 #' file. See \code{\link{read_opm}} and \code{\link{read_single_opm}} for
 #' reading such files.
 #'
-#' @param object \code{\link{OPM}} object.
+#' @param object \code{\link{OPM}} or \code{\link{OPMS}} object.
 #' @param keys Character vector (or other objects usable as vector index). An
 #'   optional sub-selection. If empty (the default), all \acronym{CSV} data are
 #'   returned. By default it is an error to select non-existing items.
@@ -793,7 +844,7 @@ setMethod("filename", OPM, function(object) {
 #'
 #' @export
 #' @family getter-functions
-#' @seealso base::strtrim base::abbreviate base::gsub
+#' @seealso base::strtrim base::abbreviate base::gsu
 #' @keywords attribute utilities character
 #' @examples
 #'
@@ -834,9 +885,10 @@ setMethod("filename", OPM, function(object) {
 #'
 setGeneric("plate_type", function(object, ...) standardGeneric("plate_type"))
 
-setMethod("plate_type", OPM, function(object, ...) {
+setMethod("plate_type", OPM, function(object, ..., normalize = FALSE,
+    subtype = FALSE) {
   plate_type(object = object@csv_data[[CSV_NAMES[["PLATE_TYPE"]]]], ...,
-    normalize = FALSE, subtype = FALSE)
+    normalize = normalize, subtype = subtype)
 }, sealed = SEALED)
 
 setMethod("plate_type", "character", function(object, full = FALSE,
@@ -1334,7 +1386,7 @@ setMethod("disc_settings", OPMD, function(object) {
 #'   \item If a non-empty list, treated as list of keys. Return value would be
 #'   the list of corresponding metadata values. Here, character vectors of
 #'   length > 1 can be used to query nested metadata lists.
-#'   \item If neither empty nor a list nior a formula (i.e. usually a character
+#'   \item If neither empty nor a list nor a formula (i.e. usually a character
 #'   or numeric vector), \code{key} is treated as a single list key. Factors are
 #'   converted to \sQuote{character} mode.
 #'   \item Formulas can also be used and are converted to a list or character or
@@ -1359,10 +1411,10 @@ setMethod("disc_settings", OPMD, function(object) {
 #'   an \code{as.labels} or \code{as.groups} argument.
 #'
 #'   Even though it is not technically impossible per se, it is usually a bad
-#'   idea to select metadata entries using numeric (positional) keys. The
-#'   problem is that, in contrast to, e.g., data frames, their is no guarantee
-#'   that metadata entries with the same name occur in the same position, even
-#'   if they belong to \code{\link{OPM}} objects within a single
+#'   idea to select metadata entries using numeric (positional) or logical keys.
+#'   The problem is that, in contrast to, e.g., data frames, their is no
+#'   guarantee that metadata entries with the same name occur in the same
+#'   position, even if they belong to \code{\link{OPM}} objects within a single
 #'   \code{\link{OPMS}} object.
 #'
 #'   Formulas passed as \code{key} argument are treated by ignoring the left
@@ -1419,97 +1471,6 @@ setMethod("metadata", WMD, function(object, key = NULL, exact = TRUE,
 ################################################################################
 
 
-#' Select a subset of the plates (or time points) [deprecated]
-#'
-#' \strong{Deprecated} function to select a subset of the plates in an
-#' \code{\link{OPMS}} object based on the content of the metadata.
-#' Alternatively, select a common subset of time points from all plates. The
-#' data-frame method selects columns that belong to certain classes.
-#'
-#' @param object \code{\link{OPMS}} object.
-#' @param query Logical, numeric or character vector, or list (other objects can
-#'   be provided but are coerced to class \sQuote{character}). If a logical or
-#'   numeric vector, \code{query} is directly used as the first argument of
-#'   \code{\link{[}}, and all following arguments, if any, are ignored. If a
-#'   list or a character vector, it is used for conducting a query based on one
-#'   of the infix operators as described below. The data-frame method expects a
-#'   character vector containing class names.
-#' @param ... Optional arguments passed to \code{\link{subset}}.
-#' @export
-#' @return \code{NULL} or \code{\link{OPM}} or \code{\link{OPMS}} object. This
-#'   depends on how many plates are selected; see \code{\link{[}} for details.
-#'   The data-frame method returns a data frame.
-#' @note The use of this function is \strong{deprecated}. Please use
-#'   \code{\link{subset}} instead.
-#'
-#' @family getter-functions
-#' @keywords manip
-#' @seealso base::`[` base::`[[` base::subset
-#' @examples
-#'
-#' ## 'OPMS' method
-#' data(vaas_4)
-#' # simple object comparison function
-#' mustbe <- function(a, b) stopifnot(identical(a, b))
-#'
-#' # all plates have that entry: selection identical to original object
-#' mustbe(vaas_4, vaas_4["Species" %k% vaas_4, ])
-#' mustbe(vaas_4, select(vaas_4, list(Species = "Escherichia coli"),
-#'   values  = FALSE)) # equivalent
-#'
-#' # two plates also have that value: yielding OPMS object with only two plates
-#' mustbe(vaas_4[1:2], vaas_4[list(Species = "Escherichia coli") %q% vaas_4, ])
-#' mustbe(vaas_4[1:2], select(vaas_4, list(Species = "Escherichia coli")))
-#'
-#' # select all plates that have aggregated values
-#' x <- select(vaas_4, has_aggr(vaas_4))
-#' mustbe(x, vaas_4) # all have such values
-#'
-#' # select a common set of time points
-#' x <- select(vaas_4, time = TRUE)
-#' mustbe(x, vaas_4) # the time points had already been identical
-#' # create unequal time points
-#' copy <- vaas_4[, list(1:10, 1:20, 1:15, 1:10)]
-#' mustbe(hours(copy), c(2.25, 4.75, 3.50, 2.25))
-#' # now restrict to common subset
-#' x <- select(copy, time = TRUE)
-#' mustbe(hours(x), rep(2.25, 4))
-#' # see also the example with split() given under "["
-#'
-#' # select all wells that have positive reactions
-#' dim(x <- select(vaas_4, use = "p")) # in at least one plate
-#' stopifnot(dim(x)[3] < dim(vaas_4)[3])
-#' dim(y <- select(vaas_4, use = "P")) # in all plates
-#' stopifnot(dim(y)[3] < dim(x)[3])
-#'
-#' # select all wells that have non-negative reactions in at least one plate
-#' dim(y <- select(vaas_4, use = "N", invert = TRUE))
-#' stopifnot(dim(y)[3] > dim(x)[3])
-#'
-#' ## data-frame method
-#' x <- data.frame(a = 1:5, b = letters[1:5], c = LETTERS[1:5])
-#' (y <- select(x, "factor"))
-#' stopifnot(dim(y) == c(5, 2))
-#' (y <- select(x, "integer"))
-#' stopifnot(dim(y) == c(5, 1))
-#' (y <- select(x, c("factor", "integer")))
-#' mustbe(x, y)
-#'
-setGeneric("select", function(object, query, ...) standardGeneric("select"))
-
-setMethod("select", OPMS, function(object, query, ...) {
-  message("select() is deprecated -- use subset() instead")
-  subset(x = object, query = query, ...)
-}, sealed = SEALED)
-
-setMethod("select", "data.frame", function(object, query) {
-  object[, vapply(object, inherits, logical(1L), what = query), drop = FALSE]
-}, sealed = SEALED)
-
-
-################################################################################
-
-
 #' Select a subset of the plates (or time points)
 #'
 #' Select a subset of the plates in an \code{\link{OPMS}} object based on the
@@ -1673,8 +1634,8 @@ setMethod("subset", OPMS, function(x, query, values = TRUE,
   if (time) {
     tp <- hours(x, what = "all")
     if (is.matrix(tp))
-      tp <- lapply(seq.int(nrow(tp)), function(i) tp[i, ])
-    if (length(maxs <- unique.default(vapply(tp, max, numeric(1L)))) < 2L)
+      tp <- lapply(seq_len(nrow(tp)), function(i) tp[i, ])
+    if (length(maxs <- unique.default(vapply(tp, max, 1))) < 2L)
       return(x)
     min.max <- min(maxs)
     tp <- lapply(tp, function(x) which(x <= min.max))
@@ -1862,10 +1823,9 @@ lapply(c(
   ), FUN = function(func_) {
   setMethod(func_, OPMS, function(object, ...) {
     simplify_conditionally <- function(x) { # instead of sapply()
-      if (any(vapply(x, is.list, logical(1L))) ||
-          any(vapply(x, is.matrix, logical(1L))))
+      if (any(vapply(x, is.list, NA)) || any(vapply(x, is.matrix, NA)))
         return(x)
-      if (length(n <- unique(vapply(x, length, integer(1L)))) > 1L)
+      if (length(n <- unique.default(vapply(x, length, 0L))) > 1L)
         return(x)
       if (n > 1L)
         do.call(rbind, x)
@@ -2298,7 +2258,7 @@ setMethod("%Q%", c("character", WMD), function(x, table) {
   if (length(keys <- names(x)) == 0L && length(x) > 0L)
     return(FALSE)
   all(vapply(keys, function(key) identical(x[[key]], table@metadata[[key]]),
-    logical(1L)))
+    NA))
 }, sealed = SEALED)
 
 setMethod("%Q%", c("list", WMD), function(x, table) {
@@ -2346,7 +2306,7 @@ lapply(c(
     #-
   ), FUN = function(func_) {
   setMethod(func_, c("list", OPMS), function(x, table) {
-    vapply(table@plates, func_, logical(1L), x = x, USE.NAMES = FALSE)
+    vapply(table@plates, func_, NA, x = x, USE.NAMES = FALSE)
   }, sealed = SEALED)
 })
 
@@ -2359,7 +2319,7 @@ lapply(c(
     #-
   ), FUN = function(func_) {
   setMethod(func_, c("character", OPMS), function(x, table) {
-    vapply(table@plates, func_, logical(1L), x = x, USE.NAMES = FALSE)
+    vapply(table@plates, func_, NA, x = x, USE.NAMES = FALSE)
   }, sealed = SEALED)
 })
 
@@ -2372,7 +2332,7 @@ lapply(c(
     #-
   ), FUN = function(func_) {
   setMethod(func_, c("factor", OPMS), function(x, table) {
-    vapply(table@plates, func_, logical(1L), x = x, USE.NAMES = FALSE)
+    vapply(table@plates, func_, NA, x = x, USE.NAMES = FALSE)
   }, sealed = SEALED)
 })
 
@@ -2385,7 +2345,7 @@ lapply(c(
     #-
   ), FUN = function(func_) {
   setMethod(func_, c("formula", OPMS), function(x, table) {
-    vapply(table@plates, func_, logical(1L), x = x, USE.NAMES = FALSE)
+    vapply(table@plates, func_, NA, x = x, USE.NAMES = FALSE)
   }, sealed = SEALED)
 })
 
